# -*- mode: perl; coding: utf-8; tab-width: 4; -*-

package Cv::StereoBMState;

use 5.008000;
use strict;
use warnings;

BEGIN {
	Cv::aliases(
		[ 'Cv::CreateStereoBMState', 'new' ],
		[ 'cvFindStereoCorrespondenceBM' ],
		);
}


1;
__END__
